#pragma once

#include <boost/spirit/include/qi.hpp>
#include <boost/optional.hpp>

namespace spirit = boost::spirit;
namespace qi = boost::spirit::qi;
namespace phx = boost::phoenix;
namespace fusion = boost::fusion;

namespace controller {

struct transition {
    std::string pre;
    std::string input;
    std::string post;
};

typedef std::vector< transition > Controller;

class ControllerParser {
public:
private:

    typedef std::string::const_iterator iterator_type;

    template <typename Iterator>
    struct transition_parser : qi::grammar<Iterator, transition(), qi::space_type> {
        qi::rule<Iterator, std::string()> quoted_string;
        qi::rule<Iterator, transition(), qi::space_type> transition_rule;

        transition_parser();
    };
    template <typename Iterator>
    struct comment_parser : qi::grammar<Iterator, void(), qi::space_type> {
        qi::rule<Iterator, void(), qi::space_type> comment_rule;

        comment_parser();
    };

    const transition_parser<iterator_type> parser;

    int parse_line(unsigned int line_number, std::string& line, transition& transition) const;

public:
    ControllerParser();
    ~ControllerParser();

    boost::optional<Controller> parse(const std::string& filename) const;
};

}
