#include "GameCreator.h"

#include <iostream>
#include <map>
#include <set>
#include <list>

namespace controller {

GameCreator::GameCreator(const Controller controller) :
    controller(controller)
{ }

GameCreator::~GameCreator() { }

mpg::MPGArena* GameCreator::createMPG() const {
    typedef std::string state_type;
    typedef std::string input_type;
    typedef std::pair< state_type, input_type > node_type;

    mpg::MPGArena* mpg_arena = new mpg::MPGArena();
    std::map< node_type, mpg::Node* > max_node_map;
    std::map< node_type, mpg::Node* > min_node_map;

    std::map< state_type, std::set< input_type > > state_in_map;
    std::map< state_type, std::set< input_type > > state_out_map;
    std::map< std::pair< state_type, input_type >, std::vector< state_type > > state_input_out_map;

    // extract information necessary for node construction
    for (const auto& transition : controller) {
        state_in_map[transition.post].insert(transition.input);
        state_out_map[transition.pre].insert(transition.input);
        state_input_out_map[{transition.pre, transition.input}].push_back(transition.post);
    }
    // add max nodes
    unsigned int i = 0;
    for (const auto& state_in_pair : state_in_map) {
        state_type s = state_in_pair.first;
        for (const input_type& x : state_in_pair.second) {
            std::string name = s + ";" + x;
            mpg::Node* node = mpg_arena->add_node(i, mpg::Player::MAX, name);
            max_node_map[{s, x}] = node;
            i++;
        }
    }
    // add min nodes and edges from min to max nodes
    for (const auto& state_input_out_pair : state_input_out_map) {
        state_type s = state_input_out_pair.first.first;
        input_type x = state_input_out_pair.first.second;
        std::string name = s + ";" + x;
        mpg::Node* node = mpg_arena->add_node(i, mpg::Player::MIN, name);
        min_node_map[{s, x}] = node;
        i++;
        for (const state_type& s_succ : state_input_out_pair.second) {
            mpg::Node* succ = max_node_map[{s_succ, x}];
            mpg_arena->add_edge(node, succ, 0);
        }
    }
    // add edges from max to min nodes
    for (const auto& max_node : max_node_map) {
        state_type s = max_node.first.first;
        state_type x_pre = max_node.first.second;
        mpg::Node* node = max_node.second;
        for (const input_type& x : state_out_map[s]) {
            mpg::Node* succ = min_node_map[{s, x}];
            mpg::weight_t weight;
            if (x_pre == x) {
                weight = 1;
            }
            else {
                weight = 0;
            }
            mpg_arena->add_edge(node, succ, weight);
        }
    }

    return mpg_arena;
}

}
