#include <iostream>
#include <fstream>
#include <vector>

#include <boost/optional.hpp>

#include "OptionParser.h"
#include "GameCreator.h"
#include "ControllerParser.h"
#include "mpg/MPGArena.h"

int main(const int argc, const char* argv[]) {

    // Parse options
    controller::OptionParser option_parser;
    auto result_options_parse = option_parser.parse_options(argc, argv);
    if (!result_options_parse) {
        return EXIT_FAILURE;
    }
    controller::Options options = *result_options_parse;

    // Validate options
    if (options.help) {
        std::cout << "Usage: " << argv[0] << " [OPTIONS] FILE" << std::endl;
        std::cout << "Convert the parity game in FILE to a mean-payoff game." << std::endl;
        std::cout << std::endl;
        option_parser.print_options();
        return EXIT_SUCCESS;
    }
    if (options.input_file.empty()) {
        std::cerr << "Error: No input file was given." << std::endl;
        return EXIT_FAILURE;
    }

    controller::ControllerParser parser;
    auto result_controller_parse = parser.parse(options.input_file);
    if (!result_controller_parse) {
        return EXIT_FAILURE;
    }
    controller::Controller controller = *result_controller_parse;

    std::streambuf* buf;
    std::ofstream of;

    if (options.output_file.empty()) {
        buf = std::cout.rdbuf();
    }
    else {
        of.open(options.output_file);
        if (!of) {
            std::cerr << "Error: Could not open output file: " << options.output_file << std::endl;
            return EXIT_FAILURE;
        }
        buf = of.rdbuf();
    }
    std::ostream out(buf);

    controller::GameCreator game_creator(controller);
    mpg::MPGArena* mpg_arena = game_creator.createMPG();

    mpg_arena->print_file(out);

    of.close();

    delete mpg_arena;

    return EXIT_SUCCESS;
}
