#pragma once

#include <climits>
#include <iostream>
#include <fstream>

#include <boost/optional.hpp>
#include <boost/intrusive/list.hpp>

namespace in = boost::intrusive;

namespace mpg {

typedef unsigned int id_t;
typedef long long weight_t;

const weight_t WEIGHT_MAX = LLONG_MAX;
const weight_t WEIGHT_MIN = LLONG_MIN;

enum class Player { MAX, MIN };
class Node;
class Edge;

typedef in::list_base_hook< in::link_mode<in::auto_unlink> > NodeHook;
typedef in::list< Node, in::base_hook<NodeHook>, in::constant_time_size<false> > NodeList;

struct edge_src_hook;
struct edge_dst_hook;
typedef in::list_base_hook< in::tag<edge_src_hook>, in::link_mode<in::auto_unlink> > EdgeSrcHook;
typedef in::list_base_hook< in::tag<edge_dst_hook>, in::link_mode<in::auto_unlink> > EdgeDstHook;
typedef in::list< Edge, in::base_hook<EdgeSrcHook>, in::constant_time_size<false> > EdgeSrcList;
typedef in::list< Edge, in::base_hook<EdgeDstHook>, in::constant_time_size<false> > EdgeDstList;

class Node : public NodeHook {
friend class Edge;
friend class MPGArena;

protected:
    Node(id_t id, Player owner, boost::optional< std::string > name);
    ~Node();

    void unlink();
    unsigned int remove_edges();

public:
    EdgeSrcList succ_edges;
    EdgeDstList pred_edges;

    const id_t id;
    const Player owner;
    const boost::optional< std::string > name;
};

class Edge : public EdgeSrcHook, public EdgeDstHook {
friend class Node;
friend class MPGArena;

protected:
    void unlink();

    Edge(Node* src, Node* dst, weight_t weight);
    ~Edge();

public:
    const Node* src;
    const Node* dst;

    const weight_t weight;
};

class MPGArena {
public:
    NodeList nodes;
    unsigned int n_nodes;
    unsigned int n_edges;

    weight_t min_weight;
    weight_t max_weight;

    MPGArena();
    ~MPGArena();

    Node* add_node(id_t id, Player owner, boost::optional< std::string > name);
    Edge* add_edge(Node* src, Node* dst, weight_t weight);

    void remove_node(Node* node);
    void remove_edge(Edge* edge);

    void print() const;
    void print_basic_info() const;
    void print_file(std::ostream& out) const;
};

}
