#pragma once

#include <boost/spirit/include/qi.hpp>
#include <boost/optional.hpp>

#include "MPGArena.h"

namespace spirit = boost::spirit;
namespace qi = boost::spirit::qi;
namespace phx = boost::phoenix;
namespace fusion = boost::fusion;

namespace mpg {

class MPGArenaParser {
public:
    struct edge {
        id_t dst;
        weight_t weight;
    };
    struct node {
        id_t id;
        Player owner;
        boost::optional< std::string > name;

        std::vector<edge> successors;
    };
private:

    typedef std::string::const_iterator iterator_type;

    template <typename Iterator>
    struct node_parser : qi::grammar<Iterator, node(), qi::space_type> {
        qi::rule<Iterator, std::string()> quoted_string;
        qi::rule<Iterator, id_t()> id_rule;
        qi::rule<Iterator, weight_t()> weight_rule;
        qi::rule<Iterator, node(), qi::space_type> node_rule;
        qi::rule<Iterator, Player()> owner_rule;
        qi::rule<Iterator, edge(), qi::space_type> edge_rule;

        node_parser();
    };
    template <typename Iterator>
    struct comment_parser : qi::grammar<Iterator, void(), qi::space_type> {
        qi::rule<Iterator, void(), qi::space_type> comment_rule;

        comment_parser();
    };

    const node_parser<iterator_type> parser;
    const comment_parser<iterator_type> comm_parser;

    int parse_line(unsigned int line_number, std::string& line, node& node) const;

public:
    MPGArenaParser();
    ~MPGArenaParser();

    boost::optional<MPGArena*> parse(const std::string& filename) const;
};

}
