#pragma once

#include <boost/program_options.hpp>
#include <boost/algorithm/string.hpp>

namespace po = boost::program_options;

namespace mpgsolver {

enum class InputFileFormat { MPG, PG, HOA, BIN };

struct Options {
    std::vector<std::string> input_files;
    InputFileFormat input_file_format;
    std::string output_file;
    std::string device;
    size_t work_group_size;
    bool solve_game;
    bool zero_mean_partition;
    bool print_arena;
    bool print_device_info;
    bool print_arena_info;
    bool profiling;
    bool timing;
    bool help;
    bool check_for_cycles;
    bool scc_partition;
    bool compact_colors;
    bool auxilliary_nodes;
    bool sort_by_owner;
    bool check_realizability;
};

class OptionParser {
private:
    po::options_description help_options_desc_;
    po::options_description hidden_options_desc_;
    po::positional_options_description positional_options_desc_;
    po::options_description options_desc_;

public:
    OptionParser();
    ~OptionParser();

    void print_options() const;
    Options parse_options(const int argc, const char* argv[]) const;
};

}
