#include "OCLModule.h"

namespace ocl {

OCLModule::OCLModule(const std::string& name, const bool executable) :
    name(name),
    executable(executable),
    references(0),
    initialized(false)
{ }

void OCLModule::init_module() {
    if (!initialized) {
        for (auto& dependency : dependencies) {
            dependency->init_module();
        }
        init();
        initialized = true;
    }
}

void OCLModule::clean_module() {
    references -= 1;
    if (initialized && references == 0) {
        initialized = false;
        clean();
        for (auto& dependency : dependencies) {
            dependency->clean_module();
        }
    }
}

void OCLModule::execute() {
    if (executable) {
        references += 1;
        init_module();
        execute_module();
        clean_module();
    }
}

void OCLModule::addDependency(OCLModule* dependency) {
    dependencies.push_back(dependency);
    dependency->references += 1;
}

void OCLModule::print_debug(std::string str) {
#ifndef NDEBUG
    std::cout << str << std::endl;
#endif
}

}
