#pragma once

#include "parser/Arena.h"
#include "parser/BINParser.h"
#include "ocl/OCLDriver.h"
#include "ocl/OCLModule.h"
#include "OCLArenaDefinitions.h"

namespace ocl {

class OCLArena : public OCLModule {
private:
    void compactColors();
    void setDriverSettings();
    void addSingleSCC();
    void computeAuxilliaryNodes();
    void sortNodes(const bool sort_by_owner);

protected:
    OCLDriver* driver;

public:
    OCLArena(OCLDriver* driver, const parser::Arena& arena);
    OCLArena(OCLDriver* driver, const parser::BINParseResult& arena);
    ~OCLArena();

    void initArena(const bool preprocess_arena, const bool scc_partition, const bool auxilliary_nodes, const bool sort_by_owner);

    const size_t n_nodes;
    const size_t n_edges;
    size_t n_colors;
    size_t n_sccs;

    size_t n_nodes_auxilliary_fwd;
    size_t n_nodes_auxilliary_bwd;

    bool has_transpose_graph;
    bool has_n_succs;
    bool has_sccs;
    const bool has_edge_weights;
    const bool has_node_colors;
    const bool has_edge_colors;
    const parser::ParityType parity_type;
    color_t neutral_color;

    // array of size n_nodes containing ids of the originally parsed nodes
    id_t* ids;
    // array of size n_nodes containing bits for owner and nodes with an auxilliary edge
    bits_t* bits;
    // array of size n_nodes+1 containing beginning of successor edges in succs, final entry is n_edges
    id_t* succs_begin;
    // if has_n_succs is set, array of size n_nodes containing number of successors for each node
    id_t* n_succs;
    // if has_transpose_graph is set, array of size n_nodes+1 containing beginning of predecessor edges in preds, final entry is n_edges
    id_t* preds_begin;
    // if has_transpose_graph is set, array of size n_nodes containing number of predecessors for each node
    id_t* n_preds;

    // array of size n_edges containing index of successor nodes
    id_t* succs;
    // if has_transpose_graph is set, array of size n_edges containing index of predecessor nodes
    id_t* preds;

    // if has_edge_weights is set, array of size n_edges containing weights of edges
    weight_t* weights;
    // if has_node_colors is set, array of size n_nodes containing colors of nodes
    // else if has_edge_colors is set, array of size n_edges containing colors of edges
    color_t* colors;

    // if has_sccs is set, array of size n_sccs+1 containing beginning index of each scc, final entry is n_nodes
    // otherwise, array of size 2 (= n_sccs + 1) with entries 0 and n_nodes
    id_t* sccs_begin;

    // array of size n_nodes. if has_sccs is set, contains the index of the scc of each node.
    // otherwise, array containing only 0
    id_t* scc_index;

    // initial node of the arena, may be undefined with a value >= n_nodes
    id_t initial_node;

    compute::vector<bits_t>* bits_buffer;

    virtual void init();
    virtual void clean();

    virtual void print(std::ostream& out) const;
    virtual void print_basic_info() const;

    void computeTransposeGraph(); // TODO make private, add option to init arena
    void computeSCCs();
    void computeNumberOfSuccessors();
};

}
