#ifndef CL_COMPILE

#pragma once

#endif

typedef cl_uint bits_t;
typedef cl_uint id_t;

typedef cl_uint color_t;
typedef cl_long weight_t;

#define COLOR_MAX           CL_UINT_MAX
#define WEIGHT_MAX          CL_LONG_MAX
#define WEIGHT_MIN          CL_LONG_MIN

// node bits
#define N_BITS_OWNER        1
#define N_BITS_SOLVED       1
#define N_BITS_LOWER_BIT    1
#define N_BITS_UPPER_BIT    1
#define N_BITS_FWD_VISITED  1
#define N_BITS_BWD_VISITED  1
#define N_BITS_FWD_AUX_NODE 1
#define N_BITS_BWD_AUX_NODE 1

#define SHIFT_OWNER         0
#define SHIFT_SOLVED        (SHIFT_OWNER     + N_BITS_OWNER)
#define SHIFT_LOWER_BIT     (SHIFT_SOLVED    + N_BITS_SOLVED)
#define SHIFT_UPPER_BIT     (SHIFT_LOWER_BIT + N_BITS_LOWER_BIT)
#define SHIFT_FWD_VISITED   (SHIFT_UPPER_BIT + N_BITS_UPPER_BIT)
#define SHIFT_BWD_VISITED   (SHIFT_FWD_VISITED + N_BITS_FWD_VISITED)
#define SHIFT_FWD_AUX_NODE  (SHIFT_BWD_VISITED + N_BITS_BWD_VISITED)
#define SHIFT_BWD_AUX_NODE  (SHIFT_FWD_AUX_NODE + N_BITS_FWD_AUX_NODE)
//#define SHIFT_              (SHIFT_BWD_AUX_NODE + N_BITS_BWD_AUX_NODE)

#define MASK_OWNER          (1 << SHIFT_OWNER)
#define MASK_SOLVED         (1 << SHIFT_SOLVED)
#define MASK_LOWER_BIT      (1 << SHIFT_LOWER_BIT)
#define MASK_UPPER_BIT      (1 << SHIFT_UPPER_BIT)
#define MASK_BITS           (MASK_LOWER_BIT | MASK_UPPER_BIT)
#define MASK_FWD_VISITED    (1 << SHIFT_FWD_VISITED)
#define MASK_BWD_VISITED    (1 << SHIFT_BWD_VISITED)
#define MASK_VISITED        (MASK_FWD_VISITED | MASK_BWD_VISITED)
#define MASK_FWD_AUX_NODE   (1 << SHIFT_FWD_AUX_NODE)
#define MASK_BWD_AUX_NODE   (1 << SHIFT_BWD_AUX_NODE)
#define MASK_AUX_NODE       (MASK_FWD_AUX_NODE | MASK_BWD_AUX_NODE)

// edge bit for auxilliary nodes
#define N_BITS_AUX_EDGE     1
#define SHIFT_AUX_EDGE      1
#define MASK_AUX_EDGE       (~(CL_UINT_MAX >> SHIFT_AUX_EDGE))
#define MASK_NO_AUX_EDGE    (CL_UINT_MAX >> SHIFT_AUX_EDGE)

// macros for accessing bits
#define PLAYER_P0                  (0 << SHIFT_OWNER)
#define PLAYER_P1                  (1 << SHIFT_OWNER)

#define SET_OWNER_P0(bits)         (bits = ((node.bits & ~MASK_OWNER) | PLAYER_P0)
#define SET_OWNER_P1(bits)         (bits = ((node.bits & ~MASK_OWNER) | PLAYER_P1)
#define IS_P0_NODE(bits)           ((bits & MASK_OWNER) == PLAYER_P0)
#define IS_P1_NODE(bits)           ((bits & MASK_OWNER) == PLAYER_P1)

#define SET_SOLVED(bits)           (bits |= MASK_SOLVED)
#define RESET_SOLVED(bits)         (bits &= ~MASK_SOLVED)
#define IS_SOLVED(bits)            ((bits & MASK_SOLVED) != 0)
#define IS_NOT_SOLVED(bits)        ((bits & MASK_SOLVED) == 0)

#define SET_LOWER_BIT(bits)        (bits |= MASK_LOWER_BIT)
#define SET_UPPER_BIT(bits)        (bits |= MASK_UPPER_BIT)
#define SET_BOTH_BITS(bits)        (bits |= MASK_BITS)
#define RESET_LOWER_BIT(bits)      (bits &= ~MASK_LOWER_BIT)
#define RESET_UPPER_BIT(bits)      (bits &= ~MASK_UPPER_BIT)
#define RESET_BOTH_BITS(bits)      (bits &= ~MASK_BITS)
#define IS_LOWER_BIT_SET(bits)     ((bits & MASK_LOWER_BIT) != 0)
#define IS_LOWER_BIT_NOT_SET(bits) ((bits & MASK_LOWER_BIT) == 0)
#define IS_UPPER_BIT_SET(bits)     ((bits & MASK_UPPER_BIT) != 0)
#define IS_UPPER_BIT_NOT_SET(bits) ((bits & MASK_UPPER_BIT) == 0)
#define ARE_BOTH_BITS_SET(bits)    ((bits & MASK_BITS) == MASK_BITS)
#define IS_EITHER_BIT_SET(bits)    ((bits & MASK_BITS) != 0)
#define IS_NEITHER_BIT_SET(bits)   ((bits & MASK_BITS) == 0)

#define SET_FWD_VISITED(bits)       (bits |= MASK_FWD_VISITED)
#define SET_BWD_VISITED(bits)       (bits |= MASK_BWD_VISITED)
#define SET_VISITED(bits)           (bits |= MASK_VISITED)
#define RESET_FWD_VISITED(bits)     (bits &= ~MASK_FWD_VISITED)
#define RESET_BWD_VISITED(bits)     (bits &= ~MASK_BWD_VISITED)
#define RESET_VISITED(bits)         (bits &= ~MASK_VISITED)
#define IS_FWD_VISITED(bits)        ((bits & MASK_FWD_VISITED) != 0)
#define IS_BWD_VISITED(bits)        ((bits & MASK_BWD_VISITED) != 0)
#define IS_VISITED(bits)            ((bits & MASK_VISITED) != 0)
#define IS_NOT_FWD_VISITED(bits)    ((bits & MASK_FWD_VISITED) == 0)
#define IS_NOT_BWD_VISITED(bits)    ((bits & MASK_BWD_VISITED) == 0)
#define IS_NOT_VISITED(bits)        ((bits & MASK_VISITED) == 0)

#define SET_FWD_AUX_NODE(bits)      (bits |= MASK_FWD_AUX_NODE)
#define SET_BWD_AUX_NODE(bits)      (bits |= MASK_BWD_AUX_NODE)
#define SET_AUX_NODE(bits)          (bits |= MASK_AUX_NODE)
#define IS_FWD_AUX_NODE(bits)       ((bits & MASK_FWD_AUX_NODE) != 0)
#define IS_BWD_AUX_NODE(bits)       ((bits & MASK_BWD_AUX_NODE) != 0)
#define IS_AUX_NODE(bits)           ((bits & MASK_AUX_NODE) != 0)
#define IS_NO_AUX_NODE(bits)        ((bits & MASK_AUX_NODE) == 0)

#define IS_AUX_EDGE(edge)           ((edge & MASK_AUX_EDGE) != 0)
#define IS_NO_AUX_EDGE(edge)        ((edge & MASK_AUX_EDGE) == 0)
#define SET_AUX_EDGE(edge)          (edge |= MASK_AUX_EDGE)
#define RESET_AUX_EDGE(edge)        (edge &= ~MASK_AUX_EDGE)
