#pragma once

#include "ocl/OCLModule.h"
#include "ocl/arena/OCLArena.h"
#include "OCLPreprocessorDefinitions.h"

namespace ocl {

class OCLPreprocessor : public OCLModule {
private:
    OCLDriver* driver;
    OCLArena* arena;

    const bool compute_scc_partition;
    cl_uint hashtable_size;

    OCLKernel* kernel_bfs_init;
    OCLKernel* kernel_bfs_fwd_iteration;
    OCLKernel* kernel_bfs_bwd_iteration;
    OCLKernel* kernel_scc_partition_init;
    OCLKernel* kernel_scc_partition_update_values;
    OCLKernel* kernel_scc_partition_update_edges;
    OCLKernel* kernel_trivial_sccs;

    compute::vector<id_t>* succs_begin_buffer;
    compute::vector<id_t>* n_succs_buffer;
    compute::vector<id_t>* preds_begin_buffer;
    compute::vector<id_t>* n_preds_buffer;
    compute::vector<id_t>* succs_buffer;
    compute::vector<id_t>* preds_buffer;
    compute::vector<id_t>* pivots_buffer;
    compute::vector<id_t>* scc_ids_buffer;
    compute::vector<scc_value>* scc_values_buffer;
    compute::vector<scc_interval>* scc_intervals_buffer;

    id_t* scc_ids;

    void bfs();
    void trivial_sccs();
    void scc_decomposition();

    inline void print_values_debug();

protected:
    void init();
    void clean();
    void execute_module();

public:
    OCLPreprocessor(OCLDriver* driver, OCLArena* arena, const bool compute_scc_partition = true);
    ~OCLPreprocessor();
    void print_partitions(std::ostream& out) const;
};

}
