#pragma once

#include "ocl/OCLModule.h"
#include "ocl/arena/OCLArena.h"
#include "ocl/solver/OCLSolver.h"
#include "ocl/solver/OCLMPGSolverDefinitions.h"

namespace ocl {

class OCLMPGSolver : public OCLSolver {
private:
    const bool check_for_cycles;
    const bool zero_mean_partition;

    OCLKernel* kernel_p_mean_partition_problem_init;
    OCLKernel* kernel_p_mean_partition_problem_update;
    OCLKernel* kernel_ergodic_partition_problem_init;
    OCLKernel* kernel_ergodic_partition_problem_update_values;
    OCLKernel* kernel_ergodic_partition_problem_update_edges;
    OCLKernel* kernel_cycle_detection_init;
    OCLKernel* kernel_cycle_detection_iteration;
    OCLKernel* kernel_mark_cycles;
    OCLKernel* kernel_propagate_cycles;
    OCLKernel* kernel_bellman_ford_init;
    OCLKernel* kernel_bellman_ford_iteration;
    OCLKernel* kernel_strategy_improvement;

    compute::vector<id_t>* succs_begin_buffer;
    compute::vector<id_t>* n_succs_buffer;
    compute::vector<id_t>* succs_buffer;
    compute::vector<weight_t>* weights_buffer;

    compute::vector<weight_t>* distance_buffer;
    compute::vector<id_t>* successor_alt_buffer;
    compute::vector<node_value>* node_value_buffer;
    compute::vector<node_interval>* node_interval_buffer;

    node_value* node_values;

    void zero_mean_partition_problem(cl_bool invert = CL_FALSE);
    void p_mean_partition_problem(cl_bool invert = CL_FALSE);
    void strategy_iteration(cl_bool invert = CL_FALSE);
    void cycle_detection();
    void bellman_ford();
    bool strategy_improvement();
    void threeway_partition_problem();
    void ergodic_partition_problem();

    inline void print_values_debug(cl_bool invert = false, bool print_min_succ = true, bool print_max_succ = true);

protected:
    void init();
    void clean();
    void execute_module();

public:
    OCLMPGSolver(OCLDriver* driver, OCLArena* arena, const bool check_for_cycles = true, const bool zero_mean_partition = false);
    ~OCLMPGSolver();

    void print_solution(std::ostream& out) const;
};

}
