#pragma once

#include "ocl/arena/OCLArena.h"
#include "ocl/solver/OCLSolver.h"
#include "ocl/solver/OCLPGSolverDefinitions.h"

namespace ocl {

class OCLPGSolver : public OCLSolver {
private:
    const bool check_realizability;

    OCLKernel* kernel_strategy_iteration_init;
    OCLKernel* kernel_strategy_iteration_update;
    OCLKernel* kernel_bellman_ford_update;
    OCLKernel* kernel_bellman_ford_init;
    OCLKernel* kernel_bellman_ford_iteration;
    OCLKernel* kernel_strategy_improvement;

    compute::vector<id_t>* succs_buffer;
    compute::vector<id_t>* succs_begin_buffer;
    compute::vector<color_t>* colors_buffer;
    compute::vector<distance_t>* distance_buffer;
    compute::vector<distance_t>* distance_prime_buffer;

    void strategy_iteration();
    bool bellman_ford(bool prime);
    bool strategy_improvement(bool prime);

    inline void print_values_debug(bool prime = false);

protected:
    void init();
    void clean();
    void execute_module();

public:
    OCLPGSolver(OCLDriver* driver, OCLArena* arena, const bool check_realizability = false);
    ~OCLPGSolver();

    void print_solution(std::ostream& out) const;
};

}
