#include "HOPGParser.h"

#include <iostream>
#include <fstream>

namespace parser {

HOPGConsumer::HOPGConsumer() {
    arena = new Arena(false, false, true, ParityType::MAX, true);
}

HOPGConsumer::~HOPGConsumer() { }

void HOPGConsumer::addState(
        unsigned int id,
        std::shared_ptr< std::string > info,
        label_expr::ptr labelExpr,
        std::shared_ptr< int_list > accSignature
) {
    cur_node.id = id;
    if (info->at(0) == '(') {
        cur_node.owner = PLAYER_1;
    }
    else {
        cur_node.owner = PLAYER_0;
    }
    cur_node.successors = std::vector< Arena::edge >();
}

void HOPGConsumer::addEdgeWithLabel(
        unsigned int stateId,
        label_expr::ptr labelExpr,
        const int_list &conjSucessors,
        std::shared_ptr< int_list > accSignature
) {
    Arena::edge cur_edge;
    cur_edge.dst = conjSucessors.front();
    if (accSignature && !accSignature->empty()) {
        cur_edge.color = accSignature->front();
    }
    else {
        cur_edge.color = COLOR_UNDEFINED;
    }
    cur_node.successors.push_back(cur_edge);
}

void HOPGConsumer::notifyEndOfState(unsigned int stateId) {
    arena->addNode(cur_node);
}

HOPGParser::HOPGParser() { }

HOPGParser::~HOPGParser() { }

Arena* HOPGParser::parse(const std::string& filename) const {
    std::ifstream in(filename);
    if (!in.is_open()) {
        throw std::invalid_argument("Could not open input file: " + filename);
    }
    HOPGConsumer* consumer = new HOPGConsumer();
    hoa::HOAConsumer::ptr consumer_ptr(consumer);
    hoa::HOAParser::parse(in, consumer_ptr, false);
    return consumer->arena;
}

}
