#pragma once

#include "ParserUtil.h"
#include "Arena.h"

namespace parser {

class MPGArenaParser {
private:
    typedef std::string::const_iterator iterator_type;

    template <typename Iterator>
    struct node_parser : qi::grammar<Iterator, Arena::node(), qi::space_type> {
        qi::rule<Iterator, std::string()> quoted_string;
        qi::rule<Iterator, id_t()> id_rule;
        qi::rule<Iterator, weight_t()> weight_rule;
        qi::rule<Iterator, color_t()> color_rule;
        qi::rule<Iterator, Arena::node(), qi::space_type> node_rule;
        qi::rule<Iterator, Arena::node(), qi::space_type> real_node_rule;
        qi::rule<Iterator, player_t()> owner_rule;
        qi::rule<Iterator, Arena::edge(), qi::space_type> edge_rule;

        node_parser();
    };
    template <typename Iterator>
    struct comment_parser : qi::grammar<Iterator, void(), qi::space_type> {
        qi::rule<Iterator, void(), qi::space_type> comment_rule;

        comment_parser();
    };

    const node_parser<iterator_type> parser;
    const comment_parser<iterator_type> comm_parser;

    bool parse_line(unsigned int line_number, std::string& line, Arena::node& node) const;

public:
    MPGArenaParser();
    ~MPGArenaParser();

    Arena* parse(const std::string& filename) const;
};

}
