#pragma once

#include <climits>

#include <boost/optional.hpp>
#include <boost/intrusive/list.hpp>

namespace in = boost::intrusive;

namespace pg {

typedef unsigned int id_t;
typedef unsigned long long parity_t;

enum class Player { ODD, EVEN };
class Node;
class Edge;

typedef in::list_base_hook< in::link_mode<in::auto_unlink> > NodeHook;
typedef in::list< Node, in::base_hook<NodeHook>, in::constant_time_size<false> > NodeList;

struct edge_src_hook;
struct edge_dst_hook;
typedef in::list_base_hook< in::tag<edge_src_hook>, in::link_mode<in::auto_unlink> > EdgeSrcHook;
typedef in::list_base_hook< in::tag<edge_dst_hook>, in::link_mode<in::auto_unlink> > EdgeDstHook;
typedef in::list< Edge, in::base_hook<EdgeSrcHook>, in::constant_time_size<false> > EdgeSrcList;
typedef in::list< Edge, in::base_hook<EdgeDstHook>, in::constant_time_size<false> > EdgeDstList;

class Node : public NodeHook {
friend class Edge;
friend class PGArena;

protected:
    Node(id_t id, Player owner, parity_t parity, boost::optional< std::string > name);
    ~Node();

    void unlink();
    unsigned int remove_edges();

public:
    EdgeSrcList succ_edges;
    EdgeDstList pred_edges;

    const id_t id;
    const Player owner;
    const parity_t parity;
    const boost::optional< std::string > name;
};

class Edge : public EdgeSrcHook, public EdgeDstHook {
friend class Node;
friend class PGArena;

protected:
    void unlink();

    Edge(Node* src, Node* dst);
    ~Edge();

public:
    const Node* src;
    const Node* dst;
};

class PGArena {
public:
    NodeList nodes;
    unsigned int n_nodes;
    unsigned int n_edges;

    parity_t max_parity;

    PGArena();
    ~PGArena();

    Node* add_node(id_t id, Player owner, parity_t parity, boost::optional< std::string > name);
    Edge* add_edge(Node* src, Node* dst);

    void remove_node(Node* node);
    void remove_edge(Edge* edge);

    void print() const;
};

}
