#include "OptionParser.h"

#include <iostream>

namespace pg2mpg {

OptionParser::OptionParser() :
    help_options_desc_("Options"),
    hidden_options_desc_(),
    positional_options_desc_(),
    options_desc_()
{
    help_options_desc_.add_options()
        ("help,h", "display this help text and exit")
        ("output-file,o", po::value<std::string>(), "output file")
    ;
    hidden_options_desc_.add_options()
        ("input-file", po::value<std::string>(), "input file")
    ;
    po::options_description options;
    options_desc_.add(help_options_desc_).add(hidden_options_desc_);
    positional_options_desc_.add("input-file", -1);
}

OptionParser::~OptionParser() { }

void OptionParser::print_options() const {
    std::cout << help_options_desc_;
}

boost::optional<Options> OptionParser::parse_options(const int argc, const char* argv[]) const {
    po::variables_map vm;

    try {
        po::store(po::command_line_parser(argc, argv).
                options(options_desc_).
                positional(positional_options_desc_).
                run(), vm);
        po::notify(vm);
    }
    catch (std::exception& e) {
        std::cerr << "Error: " << e.what() << "\n";
        return boost::none;
    }
    catch (...) {
        std::cerr << "Error: Unknown error while parsing the options." << std::endl;
        return boost::none;
    }

    Options options;

    options.help = vm.count("help") > 0;

    if (vm.count("input-file")) {
        options.input_file = vm["input-file"].as<std::string>();
    } else {
        options.input_file = "";
    }

    if (vm.count("output-file")) {
        options.output_file = vm["output-file"].as<std::string>();
    } else {
        options.output_file = "";
    }

    return options;
}

}
