#include "PGConverter.h"

#include <iostream>
#include <map>
#include <list>
#include <limits>
#include <vector>

namespace pg2mpg {

bool convertPGArenaToMPGArena(pg::PGArena* pg_arena, mpg::MPGArena* mpg_arena) {
    mpg::Node** node_map = new mpg::Node*[pg_arena->n_nodes];
    std::map< pg::parity_t, std::vector< pg::Node* > > parity_map;

    for (auto node_it = pg_arena->nodes.begin(); node_it != pg_arena->nodes.end(); ++node_it) {
        mpg::Player mpg_owner;
        if (node_it->owner == pg::Player::EVEN) {
            mpg_owner = mpg::Player::MAX;
        }
        else {
            mpg_owner = mpg::Player::MIN;
        }
        mpg::Node* mpg_node = mpg_arena->add_node(node_it->id, mpg_owner, node_it->name);
        node_map[node_it->id] = mpg_node;
        parity_map[node_it->parity].push_back(&*node_it);
    }

    mpg::weight_t odd_parity_sum  = +1;
    mpg::weight_t even_parity_sum = +1;
    std::map< pg::parity_t, std::vector< pg::Node* > >::const_iterator it = parity_map.cbegin();
    while (it != parity_map.cend()) {
        const pg::parity_t parity = it->first;
        unsigned int parity_node_count = it->second.size();
        mpg::weight_t weight;
        mpg::weight_t operand;
        if (parity % 2 == 0) {
            operand = even_parity_sum;
            weight = operand;
        }
        else {
            operand = odd_parity_sum;
            weight = -operand;
        }
        for (const auto node_it : it->second) {
            mpg::Node* src = node_map[node_it->id];
            for (const auto &edge : node_it->succ_edges) {
                mpg::Node* dst = node_map[edge.dst->id];
                mpg_arena->add_edge(src, dst, weight);
            }
        }
        it++;
        if (it != parity_map.cend()) {
            if (
                (std::numeric_limits<mpg::weight_t>::max() / parity_node_count) < operand ||
                odd_parity_sum > std::numeric_limits<mpg::weight_t>::max() - (parity_node_count * operand))
            {
                std::cerr << "Error: Weight for parity too large: " << parity << std::endl;
                return false;
            }
            if (parity % 2 == 0) {
                odd_parity_sum += parity_node_count * operand;
            }
            else {
                even_parity_sum += parity_node_count * operand;
            }
        }
    }

    delete[] node_map;

    return true;
}

}
