#include <iostream>
#include <fstream>
#include <vector>

#include <boost/optional.hpp>

#include "OptionParser.h"
#include "PGConverter.h"
#include "pg/PGArena.h"
#include "pg/PGArenaParser.h"
#include "mpg/MPGArena.h"

int main(const int argc, const char* argv[]) {

    // Parse options
    pg2mpg::OptionParser option_parser;
    auto result_options_parse = option_parser.parse_options(argc, argv);
    if (!result_options_parse) {
        return EXIT_FAILURE;
    }
    pg2mpg::Options options = *result_options_parse;

    // Validate options
    if (options.help) {
        std::cout << "Usage: " << argv[0] << " [OPTIONS] FILE" << std::endl;
        std::cout << "Convert the parity game in FILE to a mean-payoff game." << std::endl;
        std::cout << std::endl;
        option_parser.print_options();
        return EXIT_SUCCESS;
    }
    if (options.input_file.empty()) {
        std::cerr << "Error: No input file was given." << std::endl;
        return EXIT_FAILURE;
    }

    pg::PGArenaParser parser;
    auto result_arena_parse = parser.parse(options.input_file);
    if (!result_arena_parse) {
        return EXIT_FAILURE;
    }
    pg::PGArena* pg_arena = *result_arena_parse;

    std::streambuf* buf;
    std::ofstream of;

    mpg::MPGArena* mpg_arena = new mpg::MPGArena();
    bool result = pg2mpg::convertPGArenaToMPGArena(pg_arena, mpg_arena);
    if (!result) {
        return EXIT_FAILURE;
    }

    if (options.output_file.empty()) {
        buf = std::cout.rdbuf();
    }
    else {
        of.open(options.output_file);
        if (!of) {
            std::cerr << "Error: Could not open output file: " << options.output_file << std::endl;
            return EXIT_FAILURE;
        }
        buf = of.rdbuf();
    }
    std::ostream out(buf);

    mpg_arena->print_file(out);

    of.close();

    delete mpg_arena;
    delete pg_arena;

    return EXIT_SUCCESS;
}
